-- Functions for updating the General Ledger tables
-- Copyright (c) 2001 The FreeMoney Project
-- This work is released under the GPL
-- $Id: create_function_gl.sql,v 1.9 2001/09/09 07:24:51 kaare Exp $

DROP FUNCTION gl_posting_prelim(integer,integer,integer,date,decimal,decimal,text,integer,integer,integer,integer);
DROP FUNCTION gl_posting(integer,integer,integer,date,decimal,decimal,text,integer,integer,integer,integer);

CREATE FUNCTION gl_posting_prelim(integer,integer,integer,date,
  decimal,decimal,text,integer,integer,integer,integer)
  RETURNS integer AS '
  DECLARE
    p_dim1      ALIAS FOR $1;
    p_dim2      ALIAS FOR $2;
    p_dim3      ALIAS FOR $3;
    p_periodnr  ALIAS FOR $4;
    p_debit     ALIAS FOR $5;
    p_credit    ALIAS FOR $6;
    p_text      ALIAS FOR $7;
    p_amounttype ALIAS FOR $8;
    p_currency_id  ALIAS FOR $9;
    p_batchnr   ALIAS FOR $10;
    p_journalnr ALIAS FOR $11;
    v_agseq     integer;
    v_tmp       integer;
    curtime     timestamp;
  BEGIN
    curtime := ''now'';
    SELECT ag_id INTO v_agseq FROM gl_acctgrid
     WHERE dim1 = p_dim1
     AND (dim2 IS NULL AND p_dim2 IS NULL OR dim2 = p_dim2) 
     AND (dim3 IS NULL AND p_dim3 IS NULL OR dim3 = p_dim3)
     AND type = p_amounttype AND currency_id  = p_currency_id;
    IF NOT FOUND THEN
      INSERT INTO gl_acctgrid (dim1, dim2, dim3, type, currency_id )
       VALUES (p_dim1, p_dim2, p_dim3, p_amounttype, p_currency_id );
      SELECT currval(''gl_agsequence'') INTO v_agseq;
    END IF;
   INSERT INTO gl_journal (batch_id, journalnr, ag_id , entrydate, entereddr,
      enteredcr, accounteddr, accountedcr, text, accountingdate) VALUES
      (p_batchnr, p_journalnr, v_agseq, p_periodnr, p_debit, p_credit,
       p_debit, p_credit, p_text, curtime); 
    RETURN 0;
  END;
' LANGUAGE 'PLpgSQL';

COMMENT ON FUNCTION gl_posting_prelim(integer,integer,integer,date,decimal,decimal,text,integer,integer,integer,integer) IS 'Insert gl_journal for preliminary posting';

CREATE FUNCTION gl_posting(integer,integer,integer,date,
  decimal,decimal,text,integer,integer,integer,integer)
  RETURNS integer AS '
  DECLARE
    p_dim1      ALIAS FOR $1;
    p_dim2      ALIAS FOR $2;
    p_dim3      ALIAS FOR $3;
    p_periodnr  ALIAS FOR $4;
    p_debit     ALIAS FOR $5;
    p_credit    ALIAS FOR $6;
    p_text      ALIAS FOR $7;
    p_amounttype ALIAS FOR $8;
    p_currency_id  ALIAS FOR $9;
    p_batchnr   ALIAS FOR $10;
    p_journalnr ALIAS FOR $11;
    v_agseq     integer;
    v_tmp       integer;
--    v_agseq acctgrid.ag_id %rowtype;
    curtime     timestamp;
    tr_period   date;
    period_from date;
    period_to   date;
    cur_period  date;
  BEGIN
    curtime := ''now'';
    tr_period := date_trunc(''month'',p_periodnr);
    SELECT periodfrom, periodto INTO period_from, period_to FROM sy_ownercompany;
-- First acctgrid
    SELECT ag_id INTO v_agseq FROM gl_acctgrid
     WHERE dim1 = p_dim1
     AND (dim2 IS NULL AND p_dim2 IS NULL OR dim2 = p_dim2) 
     AND (dim3 IS NULL AND p_dim3 IS NULL OR dim3 = p_dim3)
     AND type = p_amounttype AND currency_id  = p_currency_id;
    IF NOT FOUND THEN
      INSERT INTO gl_acctgrid (dim1, dim2, dim3, type, currency_id )
       VALUES (p_dim1, p_dim2, p_dim3, p_amounttype, p_currency_id );
      SELECT currval(''gl_agsequence'') INTO v_agseq;
    END IF;
-- gl_balance
    SELECT 1 into v_tmp FROM gl_balance WHERE ag_id  = v_agseq and periodnr = tr_period;
    IF NOT FOUND THEN
      INSERT INTO gl_balance (ag_id, periodnr, perioddr, periodcr,
        projecttodatedr, projecttodatecr)
       VALUES (v_agseq, tr_period, p_debit, p_credit, p_debit, p_credit);
    ELSE
      UPDATE gl_balance SET perioddr = perioddr + p_debit, 
       periodcr = periodcr + p_credit,
       projecttodatedr = projecttodatedr + p_debit, 
       projecttodatecr = projecttodatecr + p_credit, updated=curtime
       WHERE ag_id  = v_agseq AND periodnr = tr_period;
    END IF;
-- Adjust beginning balance for later periods
    cur_period := tr_period + ''1 month''::timespan;
    WHILE cur_period < period_to LOOP
     SELECT 1 into v_tmp FROM gl_balance WHERE ag_id  = v_agseq and periodnr = cur_period;
     IF NOT FOUND THEN
       INSERT INTO gl_balance (ag_id, periodnr, begindr, begincr)
        VALUES (v_agseq, cur_period, p_debit, p_credit);
     ELSE
       UPDATE gl_balance SET begindr = begindr + p_debit, 
        begincr = begincr + p_credit,
        updated=curtime
        WHERE ag_id  = v_agseq AND periodnr = cur_period;
     END IF;
     cur_period = cur_period + ''1 month''::timespan;
    END LOOP;
-- Finally, the journal
    INSERT INTO gl_journal (batch_id, journalnr, ag_id, entrydate, entereddr,
      enteredcr, accounteddr, accountedcr, text, accountingdate) VALUES
      (p_batchnr, p_journalnr, v_agseq, p_periodnr, p_debit, p_credit,
       p_debit, p_credit, p_text, curtime); 
    RETURN 0;
  END;
' LANGUAGE 'PLpgSQL';

COMMENT ON FUNCTION gl_posting(integer,integer,integer,date,decimal,decimal,text,integer,integer,integer,integer) IS 'Post amount on correct gl_balance rows, insert gl_journal.';

